// include common constant variable definitions
#include "constants.as"

//--------------------------------------------------------------------------------------
void FireAmmo()
{
    // Check to see if there are already MAX_AMMO balls in the world.
    // Remove the oldest ammo to make room for the newest if necessary.

    double fOldest = GAME_STATE::AmmoQ[0].fTimeCreated;
    int nOldestIndex = 0;
    int nInactiveIndex = -1;
    for( int iAmmo = 0; iAmmo < MAX_AMMO; iAmmo++ )
    {
        if( !GAME_STATE::AmmoQ[iAmmo].bActive )
        {
            nInactiveIndex = iAmmo;
            break;
        }
        if( GAME_STATE::AmmoQ[iAmmo].fTimeCreated < fOldest )
        {
            fOldest = GAME_STATE::AmmoQ[iAmmo].fTimeCreated;
            nOldestIndex = iAmmo;
        }
    }

    if( nInactiveIndex < 0 )
    {
        GAME_STATE::AmmoQ[nOldestIndex].bActive = false;
        GAME_STATE::nAmmoCount--;
        nInactiveIndex = nOldestIndex;
    }

    int nNewAmmoIndex = nInactiveIndex;

    // Get inverse view matrix
    D3DXMATRIXA16 mInvView;
	float det = 0.0;
	D3DXMatrixInverse(mInvView, det, FirstPersonCamera::CameraGetViewMatrix());
    //D3DXMatrixInverse( &mInvView, NULL, g_Camera.GetViewMatrix() );

    // Compute initial velocity in world space from camera space
    D3DXVECTOR4 InitialVelocity( 0.0f, 0.0f, 6.0f, 0.0f );
	D3DXVec4Transform(InitialVelocity, InitialVelocity, mInvView);
    //D3DXVec4Transform( &InitialVelocity, &InitialVelocity, &mInvView );
    D3DXVECTOR4 InitialPosition( 0.0f, -0.15f, 0.0f, 1.0f );
	//D3DXVec4Transform(InitialPosition, InitialPosition, mInvView);
    D3DXVec4Transform( InitialPosition, InitialPosition, mInvView );

    AUDIO::PlayAudioCue(AUDIO::Cue_iAmmoFire);
	//PlayAudioCue( g_audioState.iAmmoFire );

    CreateAmmo( nNewAmmoIndex, InitialPosition, InitialVelocity );
}

float Vec3Dot(const D3DXVECTOR3 &in v1, const D3DXVECTOR3 &in v2)
{
	return (v1.x * v2.x) + (v1.y * v2.y) + (v1.z * v2.z);
}

float Vec3Length(const D3DXVECTOR3 &in v)
{
	return sqrt((v.x * v.x) + (v.y * v.y) + (v.z * v.z));
}

float Vec3LengthSq(const D3DXVECTOR3 &in v)
{
	return (v.x * v.x) + (v.y * v.y) + (v.z * v.z);
}

void Vec3Normalize(D3DXVECTOR3 &out dest, const D3DXVECTOR3 &in v)
{
	float length = Vec3Length(v);
	if(length != 0.0f)
	{
		float length_inv = 1.0f / length;
		dest.x = v.x * length_inv;
		dest.y = v.y * length_inv;
		dest.z = v.z * length_inv;
	}
	else
	{
		dest.x = v.x;
		dest.y = v.y;
		dest.z = v.z;
	}
}

float min(float a, float b)
{
	if(a < b) return a;
	else return b;
}

float max(float a, float b)
{
	if(a > b) return a;
	else return b;
}

//--------------------------------------------------------------------------------------
float GetDistFromWall( D3DXVECTOR3 P1, D3DXVECTOR3 P2, D3DXVECTOR3 P3, D3DXVECTOR3 N )
{
    // Plane equation: N dot (P-P3) = 0 where P3 and points P are on plane and N is plane normal 
    // Line equation: P = P1 + u(P2-P1) where points P on line pass through points P1 and P2
    // Intersection is when: N dot ((P1+u(P2-P1))-P3)=0 
    //                    or N dot (P1+u(P2-P1))=N dot P3
    //                    or u=[N dot (P3-P1)]/[N dot(P2-P1)]

    D3DXVECTOR3 P3minusP1;
    D3DXVECTOR3 P2minusP1;

    P3minusP1 = P3 - P1;
	P2minusP1 = P2 - P1;

    float a = Vec3Dot( N, P3minusP1 );
    float b = Vec3Dot( N, P2minusP1 );
    if( (b < 0.01f) && (b != 0.0f))
    {
        float u = a / b;
        D3DXVECTOR3 PonLineInterestingPlane = P1 + u * P2minusP1;
        D3DXVECTOR3 P1ToP = P1 - PonLineInterestingPlane;
        float fDist = Vec3Length( P1ToP );

        return fDist;
    }
    else
    {
        return 9999999.0f;
    }
}


//--------------------------------------------------------------------------------------
void DroidPickNewDirection( int A )
{
    D3DXMATRIXA16 mRot;
    D3DXQUATERNION q;
    float fDistFromWalls = 0.0f;
    int nAngle = ( rand(0, 360) );
    float fRotAngle = 0.0f;
    while( fDistFromWalls < 1.0f )
    {
        nAngle += 62; nAngle %= 360;
        fRotAngle = ( nAngle / 360.0f ) * PI * 2.0f;
        D3DXQuaternionRotationYawPitchRoll( q, fRotAngle, 0.0f, 0.0f );
        D3DXMatrixRotationQuaternion( mRot, q );
        D3DXVECTOR3 vVel = D3DXVECTOR3( mRot._31, mRot._32, mRot._33 );
        D3DXVECTOR3 vPos = GAME_STATE::DroidQ[A].vPosition;
        D3DXVECTOR3 vPosPlus = vPos + vVel;

        D3DXVECTOR3 vWallPtA = D3DXVECTOR3( g_MinBound.x, vPos.y, g_MinBound.z );
        D3DXVECTOR3 vWallPtB = D3DXVECTOR3( g_MaxBound.x, vPos.y, g_MinBound.z );
        D3DXVECTOR3 vWallPtC = D3DXVECTOR3( g_MinBound.x, vPos.y, g_MaxBound.z );
        D3DXVECTOR3 vWallPtD = D3DXVECTOR3( g_MaxBound.x, vPos.y, g_MaxBound.z );
        D3DXVECTOR3 vWallABNormal = D3DXVECTOR3( 0, 0, 1 );
        D3DXVECTOR3 vWallACNormal = D3DXVECTOR3( 1, 0, 0 );
        D3DXVECTOR3 vWallBDNormal = D3DXVECTOR3( 0, 0, -1 );
        D3DXVECTOR3 vWallCDNormal = D3DXVECTOR3( -1, 0, 0 );

        float fDistFromWallAB = GetDistFromWall( vPos, vPosPlus, vWallPtA, vWallABNormal );
        float fDistFromWallAC = GetDistFromWall( vPos, vPosPlus, vWallPtA, vWallACNormal );
        float fDistFromWallBD = GetDistFromWall( vPos, vPosPlus, vWallPtD, vWallBDNormal );
        float fDistFromWallCD = GetDistFromWall( vPos, vPosPlus, vWallPtD, vWallCDNormal );
        float fMin1 = min( fDistFromWallAB, fDistFromWallAC );
        float fMin2 = min( fDistFromWallBD, fDistFromWallCD );
        fDistFromWalls = min( fMin1, fMin2 );
    }

    GAME_STATE::DroidQ[A].fTargetRotation = fRotAngle;
    D3DXQuaternionRotationYawPitchRoll( GAME_STATE::DroidQ[A].qTarget, GAME_STATE::DroidQ[A].fTargetRotation, 0.0f,
                                        0.0f );
    GAME_STATE::DroidQ[A].qStart = GAME_STATE::DroidQ[A].qCurrent;
    GAME_STATE::DroidQ[A].fRotInterp = 0.0f;
    GAME_STATE::DroidQ[A].aiState = AI_TURNING;
    GAME_STATE::DroidQ[A].vVelocity = D3DXVECTOR3( 0, 0, 0 );
    GAME_STATE::DroidQ[A].vNudgeVelocity = D3DXVECTOR3( 0, 0, 0 );
}


//--------------------------------------------------------------------------------------
void DroidChooseNewTask( int A )
{
    int nRand = rand(0, 100);
    if( nRand < DROID_TURN_AI_PERCENT && GAME_STATE::DroidQ[A].aiState != AI_STOPPED )
    {
        DroidPickNewDirection( A );
    }
    else if( nRand < ( DROID_TURN_AI_PERCENT + DROID_TURN_AI_PERCENT ) || GAME_STATE::DroidQ[A].aiState == AI_STOPPED )
    {
        GAME_STATE::DroidQ[A].aiState = AI_MOVING;
        D3DXMATRIXA16 mRot;
        D3DXMatrixRotationQuaternion( mRot, GAME_STATE::DroidQ[A].qTarget );
        GAME_STATE::DroidQ[A].vVelocity = D3DXVECTOR3( mRot._31, mRot._32, mRot._33 ) * DROID_VELOCITY;
        GAME_STATE::DroidQ[A].fTaskTimer = ( rand(0, DROID_MOVE_TIME_FLUX ) + DROID_MOVE_TIME_MIN ) / 1000.0f;
    }
    else
    {
		D3DXVECTOR3 pos = GAME_STATE::DroidQ[A].vPosition;
        AUDIO::Play3DAudioCue( AUDIO::Cue_iDroidScan, pos );
        GAME_STATE::DroidQ[A].aiState = AI_STOPPED;
        GAME_STATE::DroidQ[A].fTaskTimer = 1.5f;
        GAME_STATE::DroidQ[A].fRotInterp = 0.0f;
        GAME_STATE::DroidQ[A].qStart = GAME_STATE::DroidQ[A].qCurrent;
        GAME_STATE::DroidQ[A].vVelocity = D3DXVECTOR3( 0, 0, 0 );
    }
}


//--------------------------------------------------------------------------------------
void HandleDroidAI( float fElapsedTime )
{
    for( int A = 0; A < MAX_DROID; A++ )
    {
        if( !GAME_STATE::DroidQ[A].bActive )
            continue;

        if( GAME_STATE::DroidQ[A].nHitPoints > 0 )
        {
            if( GAME_STATE::DroidQ[A].fAlpha < 1.0f )
            {
                GAME_STATE::DroidQ[A].fAlpha += fElapsedTime;
                GAME_STATE::DroidQ[A].fAlpha = min( GAME_STATE::DroidQ[A].fAlpha, 1.0f );
            }

            switch( GAME_STATE::DroidQ[A].aiState )
            {
                case AI_TURNING:
                {
                    GAME_STATE::DroidQ[A].fRotInterp += fElapsedTime;
                    if( GAME_STATE::DroidQ[A].fRotInterp > 1.0f )
                        GAME_STATE::DroidQ[A].fRotInterp = 1.0f;
                    D3DXQuaternionSlerp( GAME_STATE::DroidQ[A].qCurrent, GAME_STATE::DroidQ[A].qStart,
                                         GAME_STATE::DroidQ[A].qTarget, GAME_STATE::DroidQ[A].fRotInterp );

                    if( GAME_STATE::DroidQ[A].fRotInterp == 1.0f )
                    {
                        D3DXMATRIXA16 mRot;
                        D3DXMatrixRotationQuaternion( mRot, GAME_STATE::DroidQ[A].qCurrent );
                        GAME_STATE::DroidQ[A].vVelocity = D3DXVECTOR3( mRot._31, mRot._32, mRot._33 ) * DROID_VELOCITY;
                        GAME_STATE::DroidQ[A].aiState = AI_MOVING;
                    }
                    break;
                }

                case AI_MOVING:
                {
                    if( GAME_STATE::bDroidMove )
                        GAME_STATE::DroidQ[A].vPosition += GAME_STATE::DroidQ[A].vVelocity * fElapsedTime;

                    // Check bounce on front and back walls
                    if( GAME_STATE::DroidQ[A].vPosition.z < g_MinBound.z + ( DROID_SIZE * 0.6f ) )
                    {
                        GAME_STATE::DroidQ[A].vPosition.z = g_MinBound.z + ( DROID_SIZE * 0.6f );
                        DroidPickNewDirection( A );
                    }
                    if( GAME_STATE::DroidQ[A].vPosition.z > g_MaxBound.z - ( DROID_SIZE * 0.6f ) )
                    {
                        GAME_STATE::DroidQ[A].vPosition.z = g_MaxBound.z - ( DROID_SIZE * 0.6f );
                        DroidPickNewDirection( A );
                    }

                    // Check bounce on left and right walls
                    if( GAME_STATE::DroidQ[A].vPosition.x < g_MinBound.x + ( DROID_SIZE * 0.6f ) )
                    {
                        GAME_STATE::DroidQ[A].vPosition.x = g_MinBound.x + ( DROID_SIZE * 0.6f );
                        DroidPickNewDirection( A );
                    }
                    if( GAME_STATE::DroidQ[A].vPosition.x > g_MaxBound.x - ( DROID_SIZE * 0.6f ) )
                    {
                        GAME_STATE::DroidQ[A].vPosition.x = g_MaxBound.x - ( DROID_SIZE * 0.6f );
                        DroidPickNewDirection( A );
                    }

                    GAME_STATE::DroidQ[A].fTaskTimer -= fElapsedTime;
                    if( GAME_STATE::DroidQ[A].fTaskTimer < 0.0f )
                        DroidChooseNewTask( A );
                    break;
                }

                case AI_STOPPED:
                {
                    D3DXQUATERNION q;
                    GAME_STATE::DroidQ[A].fRotInterp += fElapsedTime * 10.0f;
                    if( GAME_STATE::DroidQ[A].fRotInterp > 4.0f * PI )
                        GAME_STATE::DroidQ[A].fRotInterp = 4.0f * PI;
                    float fRot = sin( GAME_STATE::DroidQ[A].fRotInterp ) * 0.3f;
                    D3DXQuaternionRotationYawPitchRoll( q, 0.0f, fRot, 0.0f );
                    D3DXQuaternionMultiply( GAME_STATE::DroidQ[A].qCurrent, q, GAME_STATE::DroidQ[A].qStart );

                    GAME_STATE::DroidQ[A].fTaskTimer -= fElapsedTime;
                    if( GAME_STATE::DroidQ[A].fTaskTimer < 0.0f )
                        DroidChooseNewTask( A );
                    break;
                }
            }

            // Nudge droid and keep within bounds
            GAME_STATE::DroidQ[A].vPosition += GAME_STATE::DroidQ[A].vNudgeVelocity * fElapsedTime;
            if( GAME_STATE::DroidQ[A].vPosition.z < g_MinBound.z + ( DROID_SIZE * 0.6f ) )
                GAME_STATE::DroidQ[A].vPosition.z = g_MinBound.z + ( DROID_SIZE * 0.6f );
            if( GAME_STATE::DroidQ[A].vPosition.z > g_MaxBound.z - ( DROID_SIZE * 0.6f ) )
                GAME_STATE::DroidQ[A].vPosition.z = g_MaxBound.z - ( DROID_SIZE * 0.6f );
            if( GAME_STATE::DroidQ[A].vPosition.x < g_MinBound.x + ( DROID_SIZE * 0.6f ) )
                GAME_STATE::DroidQ[A].vPosition.x = g_MinBound.x + ( DROID_SIZE * 0.6f );
            if( GAME_STATE::DroidQ[A].vPosition.x > g_MaxBound.x - ( DROID_SIZE * 0.6f ) )
                GAME_STATE::DroidQ[A].vPosition.x = g_MaxBound.x - ( DROID_SIZE * 0.6f );
            if( GAME_STATE::DroidQ[A].vPosition.y < -GROUND_Y + ( DROID_SIZE * 0.6f ) )
                GAME_STATE::DroidQ[A].vPosition.y = -GROUND_Y + ( DROID_SIZE * 0.6f );
            if( GAME_STATE::DroidQ[A].vPosition.y > GROUND_Y - ( DROID_SIZE * 0.6f ) )
                GAME_STATE::DroidQ[A].vPosition.y = GROUND_Y - ( DROID_SIZE * 0.6f );

            // Apply drag to the nudge velocity
            GAME_STATE::DroidQ[A].vNudgeVelocity -= GAME_STATE::DroidQ[A].vNudgeVelocity * ( fElapsedTime * 20.0f );
        }
        else
        {
            // Hitpoints are gone, so animate explosion
            GAME_STATE::DroidQ[A].fDeathAnimation += fElapsedTime * DROID_DEATH_SPEED;
            if( GAME_STATE::DroidQ[A].fDeathAnimation > 1.0f )
            {
                if( GAME_STATE::DroidQ[A].fAlpha > 0.0f )
                {
                    GAME_STATE::DroidQ[A].fAlpha -= fElapsedTime * DROID_DEATH_SPEED;
                    GAME_STATE::DroidQ[A].fAlpha = max( GAME_STATE::DroidQ[A].fAlpha, 0.0f );
                }
                else
                {
                    GAME_STATE::DroidQ[A].bActive = false;
                    GAME_STATE::nDroidCount--;
                    AUDIO::SetNumDroidsForAudio( GAME_STATE::nDroidCount );
                }
            }
        }
    }
}


//--------------------------------------------------------------------------------------
void CheckForAmmoToDroidCollision( int A )
{
    for( int B = 0; B < MAX_DROID; B++ )
    {
        if( !GAME_STATE::DroidQ[B].bActive || GAME_STATE::DroidQ[B].nHitPoints <= 0 )
            continue;

        // Check collision between instances A and B.
        // vAToB is the collision normal vector.
        D3DXVECTOR3 vAToB = - GAME_STATE::AmmoQ[A].vPosition + GAME_STATE::DroidQ[B].vPosition;
        float DistSq = Vec3LengthSq( vAToB );
        if( DistSq < ( ( AMMO_SIZE + DROID_SIZE ) / 2.0f ) * ( ( AMMO_SIZE + DROID_SIZE ) / 2.0f ) )
        {
            Vec3Normalize( vAToB, vAToB );

            // Check if the B instances are already moving away from each other.
            // If so, skip collision.  This can happen when a lot of instances are
            // bunched up next to each other.
            float fImpact = Vec3Dot( vAToB, GAME_STATE::AmmoQ[A].vVelocity ) -
                Vec3Dot( vAToB, GAME_STATE::DroidQ[B].vVelocity );
            if( fImpact > 0.0f )
            {
                GAME_STATE::AmmoQ[A].bActive = false;
                GAME_STATE::DroidQ[B].vNudgeVelocity += GAME_STATE::AmmoQ[A].vVelocity * 0.3f;

                GAME_STATE::DroidQ[B].nHitPoints--;
                if( GAME_STATE::DroidQ[B].nHitPoints <= 0 )
                {
                    GAME_STATE::DroidQ[B].fDeathAnimation = 0.0f;
                    AUDIO::Play3DAudioCue( AUDIO::Cue_iDroidDestroyed, GAME_STATE::DroidQ[B].vPosition );
                }
            }
        }
    }
}


//--------------------------------------------------------------------------------------
void CheckForInterAmmoCollision( float fElapsedTime )
{
    if( GAME_STATE::nAmmoCount <= 1 )
        return;

    for( int A = 0; A < MAX_AMMO - 1; A++ )
    {
        if( !GAME_STATE::AmmoQ[A].bActive )
            continue;

        for( int B = A + 1; B < MAX_AMMO; B++ )
        {
            if( !GAME_STATE::AmmoQ[B].bActive )
                continue;

            // Check collision between instances A and B.
            // vAToB is the collision normal vector.
            D3DXVECTOR3 vAToB = GAME_STATE::AmmoQ[B].vPosition - GAME_STATE::AmmoQ[A].vPosition;
            float DistSq = Vec3LengthSq( vAToB );
            if( DistSq < AMMO_SIZE * AMMO_SIZE )
            {
                Vec3Normalize( vAToB, vAToB );

                // Check if the B instances are already moving away from each other.
                // If so, skip collision.  This can happen when a lot of instances are
                // bunched up next to each other.
                float fImpact = Vec3Dot( vAToB, GAME_STATE::AmmoQ[A].vVelocity ) -
                    Vec3Dot( vAToB, GAME_STATE::AmmoQ[B].vVelocity );
                if( fImpact > 0.0f )
                {
                    // Compute the normal and tangential compAnts of A's velocity.
                    D3DXVECTOR3 vVelocityAN = ( 1 - BOUNCE_LOST ) *
                        Vec3Dot( vAToB, GAME_STATE::AmmoQ[A].vVelocity ) * vAToB;
                    D3DXVECTOR3 vVelocityAT = ( 1 - BOUNCE_LOST ) * GAME_STATE::AmmoQ[A].vVelocity - vVelocityAN;

                    // Compute the normal and tangential compAnts of B's velocity.
                    D3DXVECTOR3 vVelocityBN = ( 1 - BOUNCE_LOST ) *
                        Vec3Dot( vAToB, GAME_STATE::AmmoQ[B].vVelocity ) * vAToB;
                    D3DXVECTOR3 vVelocityBT = ( 1 - BOUNCE_LOST ) * GAME_STATE::AmmoQ[B].vVelocity - vVelocityBN;

                    // Compute post-collision velocity
                    GAME_STATE::AmmoQ[A].vVelocity = vVelocityAT - vVelocityAN * ( 1 - BOUNCE_TRANSFER ) +
                        vVelocityBN * BOUNCE_TRANSFER;
                    GAME_STATE::AmmoQ[B].vVelocity = vVelocityBT - vVelocityBN * ( 1 - BOUNCE_TRANSFER ) +
                        vVelocityAN * BOUNCE_TRANSFER;

                    // Fix the positions so that the B balls are exactly AMMO_SIZE apart.
                    float fDistanceToMove = ( AMMO_SIZE - sqrt( DistSq ) ) * 0.5f;
                    GAME_STATE::AmmoQ[A].vPosition -= vAToB * fDistanceToMove;
                    GAME_STATE::AmmoQ[B].vPosition += vAToB * fDistanceToMove;

                    // Flag the B instances so that they are not laying on ground.
                    GAME_STATE::AmmoQ[A].bGround = GAME_STATE::AmmoQ[B].bGround = false;

                    // Play sound
                    //                  GAME_STATE::AmmoQ[A].PlaySound( fImpact );
                    //                  GAME_STATE::AmmoQ[B].PlaySound( fImpact );
                }
            }
        }
    }
}


//--------------------------------------------------------------------------------------
void CheckForAmmoToWallCollision( int A )
{
    // Check bounce on ground
    if( !GAME_STATE::AmmoQ[A].bGround )
    {
        if( GAME_STATE::AmmoQ[A].vPosition.y < -GROUND_Y + ( AMMO_SIZE * 0.5f ) )
        {
            GAME_STATE::AmmoQ[A].vPosition.y = -GROUND_Y + ( AMMO_SIZE * 0.5f );
            GAME_STATE::AmmoQ[A].vVelocity.y = -GAME_STATE::AmmoQ[A].vVelocity.y * ( 1 - GROUND_ABSORBANCE );
            //GAME_STATE::AmmoQ[A].PlaySound( -GAME_STATE::AmmoQ[A].vVelocity.y );

            // X and Z velocity are reduced because of friction.
            GAME_STATE::AmmoQ[A].vVelocity.x *= 0.9f;
            GAME_STATE::AmmoQ[A].vVelocity.z *= 0.9f;
            AUDIO::PlayAudioCue( AUDIO::Cue_iAmmoBounce );
        }
    }
    else
    {
        // Ball is resting or rolling on ground.
        // X and Z velocity are reduced because of friction.
        GAME_STATE::AmmoQ[A].vVelocity.x *= 0.9f;
        GAME_STATE::AmmoQ[A].vVelocity.z *= 0.9f;
    }

    // Check bounce on ceiling
    if( GAME_STATE::AmmoQ[A].vPosition.y > GROUND_Y - ( AMMO_SIZE * 0.5f ) )
    {
        GAME_STATE::AmmoQ[A].vPosition.y = GROUND_Y - ( AMMO_SIZE * 0.5f );
        GAME_STATE::AmmoQ[A].vVelocity.y = -GAME_STATE::AmmoQ[A].vVelocity.y * ( 1 - GROUND_ABSORBANCE );
        //GAME_STATE::AmmoQ[A].PlaySound( GAME_STATE::AmmoQ[A].vVelocity.y );

        // X and Z velocity are reduced because of friction.
        GAME_STATE::AmmoQ[A].vVelocity.x *= 0.9f;
        GAME_STATE::AmmoQ[A].vVelocity.z *= 0.9f;
        AUDIO::PlayAudioCue( AUDIO::Cue_iAmmoBounce );
    }

    // If the Y direction motion is below a certain threshold, flag the instance as laying on the ground.
    if( GRAVITY * ( GAME_STATE::AmmoQ[A].vPosition.y + GROUND_Y - ( AMMO_SIZE * 0.5f ) ) + 0.5f *
        GAME_STATE::AmmoQ[A].vVelocity.y * GAME_STATE::AmmoQ[A].vVelocity.y < REST_THRESHOLD )
    {
        GAME_STATE::AmmoQ[A].vPosition.y = -GROUND_Y + ( AMMO_SIZE * 0.5f );
        GAME_STATE::AmmoQ[A].vVelocity.y = 0.0f;
        GAME_STATE::AmmoQ[A].bGround = true;
    }

    // Check bounce on front and back walls
    if( GAME_STATE::AmmoQ[A].vPosition.z < g_MinBound.z + ( AMMO_SIZE * 0.5f ) )
    {
        GAME_STATE::AmmoQ[A].vPosition.z = g_MinBound.z + ( AMMO_SIZE * 0.5f );
        GAME_STATE::AmmoQ[A].vVelocity.z = -GAME_STATE::AmmoQ[A].vVelocity.z * ( 1 - GROUND_ABSORBANCE );
        AUDIO::PlayAudioCue( AUDIO::Cue_iAmmoBounce );
    }
    if( GAME_STATE::AmmoQ[A].vPosition.z > g_MaxBound.z - ( AMMO_SIZE * 0.5f ) )
    {
        GAME_STATE::AmmoQ[A].vPosition.z = g_MaxBound.z - ( AMMO_SIZE * 0.5f );
        GAME_STATE::AmmoQ[A].vVelocity.z = -GAME_STATE::AmmoQ[A].vVelocity.z * ( 1 - GROUND_ABSORBANCE );
        AUDIO::PlayAudioCue( AUDIO::Cue_iAmmoBounce );
    }

    // Check bounce on left and right walls
    if( GAME_STATE::AmmoQ[A].vPosition.x < g_MinBound.x + ( AMMO_SIZE * 0.5f ) )
    {
        GAME_STATE::AmmoQ[A].vPosition.x = g_MinBound.x + ( AMMO_SIZE * 0.5f );
        GAME_STATE::AmmoQ[A].vVelocity.x = -GAME_STATE::AmmoQ[A].vVelocity.x * ( 1 - GROUND_ABSORBANCE );
        AUDIO::PlayAudioCue( AUDIO::Cue_iAmmoBounce );
    }
    if( GAME_STATE::AmmoQ[A].vPosition.x > g_MaxBound.x - ( AMMO_SIZE * 0.5f ) )
    {
        GAME_STATE::AmmoQ[A].vPosition.x = g_MaxBound.x - ( AMMO_SIZE * 0.5f );
        GAME_STATE::AmmoQ[A].vVelocity.x = -GAME_STATE::AmmoQ[A].vVelocity.x * ( 1 - GROUND_ABSORBANCE );
        AUDIO::PlayAudioCue( AUDIO::Cue_iAmmoBounce );
    }
}


//--------------------------------------------------------------------------------------
void HandleAmmoAI( float fElapsedTime )
{
    if( GAME_STATE::nAmmoCount == 0 )
        return;

    CheckForInterAmmoCollision( fElapsedTime );

    // Apply gravity and check for collision against ground and walls.
    for( int A = 0; A < MAX_AMMO; A++ )
    {
        if( !GAME_STATE::AmmoQ[A].bActive )
            continue;

        GAME_STATE::AmmoQ[A].fLifeCountdown -= fElapsedTime;
        if( GAME_STATE::AmmoQ[A].fLifeCountdown < 1.0f / 0.5f )
        {
            GAME_STATE::AmmoQ[A].fAlpha = GAME_STATE::AmmoQ[A].fLifeCountdown * 0.5f;
        }

        if( GAME_STATE::AmmoQ[A].fLifeCountdown < 0.0f )
        {
            GAME_STATE::AmmoQ[A].bActive = false;
            GAME_STATE::nAmmoCount--;
            continue;
        }

        // If the elapsed time is too long, we slice up the time and
        // handle physics over several passes.
        float fTimeLeft = fElapsedTime;
        float fElapsedFrameTime;

        while( fTimeLeft > 0.0f )
        {
            fElapsedFrameTime = min( fTimeLeft, PHYSICS_FRAMELENGTH );
            fTimeLeft -= fElapsedFrameTime;

            GAME_STATE::AmmoQ[A].vPosition += GAME_STATE::AmmoQ[A].vVelocity * fElapsedFrameTime;

            GAME_STATE::AmmoQ[A].vVelocity.x -= GAME_STATE::AmmoQ[A].vVelocity.x * 0.1f * fElapsedFrameTime;
            GAME_STATE::AmmoQ[A].vVelocity.z -= GAME_STATE::AmmoQ[A].vVelocity.z * 0.1f * fElapsedFrameTime;

            // Apply gravity if the ball is not resting on the ground.
            if( !GAME_STATE::AmmoQ[A].bGround )
                GAME_STATE::AmmoQ[A].vVelocity.y -= GRAVITY * fElapsedFrameTime;

            CheckForAmmoToWallCollision( A );
            CheckForAmmoToDroidCollision( A );
        }
    }
}

//--------------------------------------------------------------------------------------
void CreateAmmo( int nIndex, D3DXVECTOR4 Pos, D3DXVECTOR4 Vel )
{
    GAME_STATE::AmmoQ[nIndex].SetRandomRotation();

    GAME_STATE::AmmoQ[nIndex].bActive = true;
    GAME_STATE::AmmoQ[nIndex].fTimeCreated = DXUTGetTime();
    GAME_STATE::AmmoQ[nIndex].fLifeCountdown = 5.0f;
    GAME_STATE::AmmoQ[nIndex].fAlpha = 1.0f;
    GAME_STATE::AmmoQ[nIndex].bGround = false;
    GAME_STATE::AmmoQ[nIndex].vPosition.x = Pos.x;
    GAME_STATE::AmmoQ[nIndex].vPosition.y = Pos.y;
    GAME_STATE::AmmoQ[nIndex].vPosition.z = Pos.z;
    GAME_STATE::AmmoQ[nIndex].vVelocity.x = Vel.x;
    GAME_STATE::AmmoQ[nIndex].vVelocity.y = Vel.y;
    GAME_STATE::AmmoQ[nIndex].vVelocity.z = Vel.z;
    GAME_STATE::nAmmoCount++;

    D3DXCOLOR cBlendColor;
    D3DXColorLerp( cBlendColor, GAME_STATE::BlendFromColor, GAME_STATE::BlendToColor, GAME_STATE::fAmmoColorLerp );
    GAME_STATE::AmmoQ[nIndex].Diffuse.x = cBlendColor.r;
    GAME_STATE::AmmoQ[nIndex].Diffuse.y = cBlendColor.g;
    GAME_STATE::AmmoQ[nIndex].Diffuse.z = cBlendColor.b;
    GAME_STATE::AmmoQ[nIndex].Diffuse.w = cBlendColor.a;
}

//--------------------------------------------------------------------------------------
void CreateDroid()
{
    int A;
    for( A = 0; A < MAX_DROID; ++A )
    {
        if( !GAME_STATE::DroidQ[A].bActive )
            break;
    }
    if( A == MAX_DROID )
        return;

    D3DXVECTOR3 Pos;

    for(; ; )
    {
        float fX = ( rand(0, 100) ) / 100.0f;
        float fZ = ( rand(0, 100) ) / 100.0f;

        Pos = D3DXVECTOR3( g_MinBound.x + fX * ( g_MaxBound.x - g_MinBound.x ),
                           -GROUND_Y + DROID_MIN_HEIGHT + ( ( rand(0, 100) ) / 100.0f ) * DROID_HEIGHT_FLUX,
                           g_MinBound.z + fZ * ( g_MaxBound.z - g_MinBound.z ) );

        // Position the droids apart from each other
        bool bInterset = false;
        for( int B = 0; B < MAX_DROID; B++ )
        {
            if( !GAME_STATE::DroidQ[B].bActive )
                continue;
            D3DXVECTOR3 vAToB = Pos - GAME_STATE::DroidQ[B].vPosition;
            float Dist = Vec3Length( vAToB );
            if( Dist < 2.0f * DROID_SIZE )
            {
                bInterset = true;
                break;
            }
        }

        if( !bInterset ) break;
    }

    GAME_STATE::DroidQ[A].bActive = true;
    GAME_STATE::DroidQ[A].vPosition = Pos;

    float fStartRotation = ( rand(0,360) ) / 360.0f * PI * 2.0f;
    GAME_STATE::DroidQ[A].fTargetRotation = fStartRotation;
    D3DXQuaternionRotationYawPitchRoll( GAME_STATE::DroidQ[A].qTarget, GAME_STATE::DroidQ[A].fTargetRotation, 0.0f,
                                        0.0f );
    GAME_STATE::DroidQ[A].qStart = GAME_STATE::DroidQ[A].qTarget;
    GAME_STATE::DroidQ[A].qCurrent = GAME_STATE::DroidQ[A].qTarget;
    GAME_STATE::DroidQ[A].fRotInterp = 0.0f;
    GAME_STATE::DroidQ[A].aiState = AI_MOVING;
    GAME_STATE::DroidQ[A].fTaskTimer = 0.0f;
    GAME_STATE::DroidQ[A].nHitPoints = DROID_HITPOINTS;
    GAME_STATE::DroidQ[A].fDeathAnimation = 0.0f;
    GAME_STATE::DroidQ[A].fAlpha = 0.0f;
    D3DXMATRIXA16 mRot;
    D3DXMatrixRotationQuaternion( mRot, GAME_STATE::DroidQ[A].qTarget );
    GAME_STATE::DroidQ[A].vVelocity = D3DXVECTOR3( mRot._31, mRot._32, mRot._33 ) * DROID_VELOCITY;

    // Give the droids different specular tints
    float C1 = 0.75f;
    float C2 = 0.35f;
    switch( rand(0, 7) )
    {
        case 0:
            GAME_STATE::DroidQ[A].Specular = D3DXCOLOR( C1, C1, C2, 1.0f ); break;
        case 1:
            GAME_STATE::DroidQ[A].Specular = D3DXCOLOR( C1, C2, C1, 1.0f ); break;
        case 2:
            GAME_STATE::DroidQ[A].Specular = D3DXCOLOR( C1, C2, C2, 1.0f ); break;
        case 3:
            GAME_STATE::DroidQ[A].Specular = D3DXCOLOR( C2, C1, C1, 1.0f ); break;
        case 4:
            GAME_STATE::DroidQ[A].Specular = D3DXCOLOR( C2, C1, C2, 1.0f ); break;
        case 5:
            GAME_STATE::DroidQ[A].Specular = D3DXCOLOR( C2, C2, C1, 1.0f ); break;
        case 6:
            GAME_STATE::DroidQ[A].Specular = D3DXCOLOR( C2, C2, C2, 1.0f ); break;
    }

    GAME_STATE::nDroidCount++;
    AUDIO::SetNumDroidsForAudio( GAME_STATE::nDroidCount );
}
